@@ -22,7 +22,10 @@ from subprocess import CalledProcessError
 # Protect from scanadf[0] and possibly other brain-dead software that set
 # SIGCHLD to SIG_IGN.
 # [0] http://bugs.debian.org/596232
-signal.signal(signal.SIGCHLD, signal.SIG_DFL)
+try:
+    signal.signal(signal.SIGCHLD, signal.SIG_DFL)
+except AttributeError:
+    pass
 
 def get_signal_names():
     _signame_pattern = re.compile('^SIG[A-Z0-9]*$')
@@ -49,9 +52,11 @@ class CalledProcessInterrupted(CalledProcessError):
 
     def __init__(self, signal_id, command):
         Exception.__init__(self, command, signal_id)
+        self.by_user = signal_id == signal.SIGINT
+
     def __str__(self):
         signal_name = self._signal_names.get(self.args[1], self.args[1])
-        return 'Command %r was interrputed by signal %s' % (self.args[0], signal_name)
+        return 'Command %r was interrupted by signal %s' % (self.args[0], signal_name)
 
 del get_signal_names
 
@@ -67,7 +72,7 @@ class Subprocess(subprocess.Popen):
         env = dict(
                 (k, v)
                 for k, v in env.iteritems()
-                if not (k.startswith('LC_') or k in ('LANG', 'LANGUAGES'))
+                if not (k.startswith('LC_') or k in ('LANG', 'LANGUAGE'))
         )
         if lc_ctype:
             env['LC_CTYPE'] = lc_ctype
